@@ -1519,6 +1519,11 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 		fmt->info.raw.rate = rrate;
 		match = false;
 	}
+	if (rchannels == 0 || rrate == 0) {
+		spa_log_error(state->log, "%s: invalid channels:%d or rate:%d",
+				state->props.device, rchannels, rrate);
+		return -EIO;
+	}
 
 	state->format = rformat;
 	state->channels = rchannels;
@@ -1563,6 +1568,11 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 
 	CHECK(snd_pcm_hw_params_set_period_size_near(hndl, params, &period_size, &dir), "set_period_size_near");
 
+	if (period_size == 0) {
+		spa_log_error(state->log, "%s: invalid period_size 0 (driver error?)", state->props.device);
+		return -EIO;
+	}
+
 	state->period_frames = period_size;
 
 	if (state->default_period_num != 0) {
@@ -1578,16 +1588,27 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 		CHECK(snd_pcm_hw_params_set_buffer_size_near(hndl, params, &state->buffer_frames), "set_buffer_size_near");
 		periods = state->buffer_frames / period_size;
 	}
+	if (state->buffer_frames == 0) {
+		spa_log_error(state->log, "%s: invalid buffer_frames 0 (driver error?)", state->props.device);
+		return -EIO;
+	}
 
 	state->headroom = state->default_headroom;
 	if (is_batch)
 		state->headroom += period_size;
 
+	if (spa_strstartswith(state->props.device, "a52") ||
+	    spa_strstartswith(state->props.device, "dca"))
+		state->min_delay = SPA_MIN(2048u, state->buffer_frames);
+	else
+		state->min_delay = 0;
+
 	state->headroom = SPA_MIN(state->headroom, state->buffer_frames);
 	state->start_delay = state->default_start_delay;
 
-	state->latency[state->port_direction].min_rate = state->headroom;
-	state->latency[state->port_direction].max_rate = state->headroom;
+	state->latency[state->port_direction].min_rate =
+		state->latency[state->port_direction].max_rate =
+			SPA_MAX(state->min_delay, state->headroom);
 
 	spa_log_info(state->log, "%s (%s): format:%s access:%s-%s rate:%d channels:%d "
 			"buffer frames %lu, period frames %lu, periods %u, frame_size %zd "
@@ -1858,8 +1879,10 @@ static int get_status(struct state *state, uint64_t current_time,
 	} else {
 		*delay = avail;
 		*target = SPA_MAX(*target, state->read_size);
+		if (state->matching)
+			*target += 32;
 	}
-	*target = SPA_MIN(*target, state->buffer_frames);
+	*target = SPA_CLAMP(*target, state->min_delay, state->buffer_frames);
 	return 0;
 }
 
@@ -1978,7 +2001,7 @@ static inline void check_position_config(struct state *state)
 	    (state->rate_denom != state->position->clock.rate.denom))) {
 		state->duration = state->position->clock.duration;
 		state->rate_denom = state->position->clock.rate.denom;
-		state->threshold = (state->duration * state->rate + state->rate_denom-1) / state->rate_denom;
+		state->threshold = SPA_SCALE32_UP(state->duration, state->rate, state->rate_denom);
 		state->max_error = SPA_MAX(256.0f, state->threshold / 2.0f);
 		state->resample = ((uint32_t)state->rate != state->rate_denom) || state->matching;
 		state->alsa_sync = true;
@@ -2428,9 +2451,20 @@ static void alsa_on_timeout_event(struct spa_source *source)
 	struct state *state = source->data;
 	snd_pcm_uframes_t delay, target;
 	uint64_t expire, current_time;
+	int res;
 
-	if (SPA_UNLIKELY(state->started && spa_system_timerfd_read(state->data_system, state->timerfd, &expire) < 0))
-		spa_log_warn(state->log, "%p: error reading timerfd: %m", state);
+	if (SPA_LIKELY(state->started)) {
+		if (SPA_UNLIKELY((res = spa_system_timerfd_read(state->data_system,
+						state->timerfd, &expire)) < 0)) {
+			/* we can get here when the timer is changed since the last
+			 * timerfd wakeup, for example by do_reassign_follower() executed
+			 * in the same epoll wakeup cycle */
+			if (res != -EAGAIN)
+				spa_log_warn(state->log, "%p: error reading timerfd: %s",
+						state, spa_strerror(res));
+			return;
+		}
+	}
 
 	check_position_config(state);
 
@@ -2524,12 +2558,20 @@ int spa_alsa_start(struct state *state)
 		state->duration = 1024;
 		state->rate_denom = state->rate;
 	}
+	if (state->rate_denom == 0) {
+		spa_log_error(state->log, "%s: unset rate_denom", state->props.device);
+		return -EIO;
+	}
+	if (state->duration == 0) {
+		spa_log_error(state->log, "%s: unset duration", state->props.device);
+		return -EIO;
+	}
 
 	state->following = is_following(state);
 	setup_matching(state);
 
 	spa_dll_init(&state->dll);
-	state->threshold = (state->duration * state->rate + state->rate_denom-1) / state->rate_denom;
+	state->threshold = SPA_SCALE32_UP(state->duration, state->rate, state->rate_denom);
 	state->last_threshold = state->threshold;
 	state->max_error = SPA_MAX(256.0f, state->threshold / 2.0f);
 
@@ -2610,6 +2652,8 @@ int spa_alsa_reassign_follower(struct state *state)
 		else
 			snd_pcm_pause(state->hndl, 0);
 	}
+
+	state->alsa_sync_warning = false;
 	return 0;
 }
 
