@@ -49,6 +49,7 @@
 #include <spa/param/bluetooth/audio.h>
 #include <spa/param/bluetooth/type-info.h>
 #include <spa/debug/pod.h>
+#include <spa/debug/log.h>
 
 #include "defs.h"
 #include "media-codecs.h"
@@ -203,6 +204,33 @@ static const struct media_codec *get_supported_media_codec(struct impl *this, en
 	return media_codec;
 }
 
+static bool is_bap_client(struct impl *this)
+{
+	struct spa_bt_device *device = this->bt_dev;
+	struct spa_bt_transport *t;
+
+	spa_list_for_each(t, &device->transport_list, device_link) {
+		if (t->bap_initiator)
+			return true;
+	}
+
+	return false;
+}
+
+static bool can_bap_codec_switch(struct impl *this)
+{
+	if (!is_bap_client(this))
+		return false;
+
+	/* XXX: codec switching for source/duplex is not currently
+	 * XXX: implemented properly. TODO: fix this
+	 */
+	if (this->bt_dev->connected_profiles & SPA_BT_PROFILE_BAP_SOURCE)
+		return false;
+
+	return true;
+}
+
 static unsigned int get_hfp_codec(enum spa_bluetooth_audio_codec id)
 {
 	switch (id) {
@@ -872,9 +900,15 @@ static int set_profile(struct impl *this, uint32_t profile, enum spa_bluetooth_a
 	 * A2DP/BAP: ensure there's a transport with the selected codec (0 means any).
 	 * Don't try to switch codecs when the device is in the A2DP source role, since
 	 * devices do not appear to like that.
+	 *
+	 * For BAP, only BAP client can configure the codec.
+	 *
+	 * XXX: codec switching also currently does not work in the duplex or
+	 * XXX: source-only case, as it will only switch the sink, and we only
+	 * XXX: list the sink codecs here. TODO: fix this
 	 */
-	if ((profile == DEVICE_PROFILE_A2DP || profile == DEVICE_PROFILE_BAP)
-	    && !(this->bt_dev->connected_profiles & SPA_BT_PROFILE_A2DP_SOURCE)) {
+	if ((profile == DEVICE_PROFILE_A2DP || (profile == DEVICE_PROFILE_BAP && can_bap_codec_switch(this)))
+			&& !(this->bt_dev->connected_profiles & SPA_BT_PROFILE_A2DP_SOURCE)) {
 		int ret;
 		const struct media_codec *codecs[64];
 
@@ -1150,13 +1184,20 @@ static uint32_t get_index_from_profile(struct impl *this, uint32_t profile, enum
 	if (profile == DEVICE_PROFILE_OFF || profile == DEVICE_PROFILE_AG)
 		return profile;
 
-	if (profile == DEVICE_PROFILE_A2DP || profile == DEVICE_PROFILE_BAP) {
+	if (profile == DEVICE_PROFILE_A2DP) {
 		if (codec == 0 || (this->bt_dev->connected_profiles & SPA_BT_PROFILE_MEDIA_SOURCE))
 			return profile;
 
 		return codec + DEVICE_PROFILE_LAST;
 	}
 
+	if (profile == DEVICE_PROFILE_BAP) {
+		if (codec == 0)
+			return profile;
+
+		return codec + DEVICE_PROFILE_LAST;
+	}
+
 	if (profile == DEVICE_PROFILE_HSP_HFP) {
 		if (codec == 0 || (this->bt_dev->connected_profiles & SPA_BT_PROFILE_HFP_AG))
 			return profile;
@@ -1324,11 +1365,6 @@ static struct spa_pod *build_profile(struct impl *this, struct spa_pod_builder *
 		if (profile == 0)
 			return NULL;
 
-		if (!codec) {
-			errno = EINVAL;
-			return NULL;
-		}
-
 		if (profile & (SPA_BT_PROFILE_BAP_SINK))
 			n_sink++;
 		if (profile & (SPA_BT_PROFILE_BAP_SOURCE))
@@ -1336,28 +1372,51 @@ static struct spa_pod *build_profile(struct impl *this, struct spa_pod_builder *
 
 		name = spa_bt_profile_name(profile);
 
-		media_codec = get_supported_media_codec(this, codec, &idx);
-		if (media_codec == NULL) {
-			errno = EINVAL;
+		/* If we can't codec switch, emit codecless profile */
+		if (current && !can_bap_codec_switch(this)) {
+			codec = 0;
+			index = get_index_from_profile(this, profile_index, codec);
+		} else if ((codec != 0) != can_bap_codec_switch(this)) {
+			errno = -EINVAL;
 			return NULL;
 		}
-		name_and_codec = spa_aprintf("%s-%s", name, media_codec->name);
-		name = name_and_codec;
-		switch (profile) {
-		case SPA_BT_PROFILE_BAP_SINK:
-			desc_and_codec = spa_aprintf(_("High Fidelity Playback (BAP Sink, codec %s)"),
-						     media_codec->description);
-			break;
-		case SPA_BT_PROFILE_BAP_SOURCE:
-			desc_and_codec = spa_aprintf(_("High Fidelity Input (BAP Source, codec %s)"),
-						     media_codec->description);
-			break;
-		default:
-			desc_and_codec = spa_aprintf(_("High Fidelity Duplex (BAP Source/Sink, codec %s)"),
-						     media_codec->description);
+
+		if (codec) {
+			media_codec = get_supported_media_codec(this, codec, &idx);
+			if (media_codec == NULL) {
+				errno = EINVAL;
+				return NULL;
+			}
+			name_and_codec = spa_aprintf("%s-%s", name, media_codec->name);
+			name = name_and_codec;
+			switch (profile) {
+			case SPA_BT_PROFILE_BAP_SINK:
+				desc_and_codec = spa_aprintf(_("High Fidelity Playback (BAP Sink, codec %s)"),
+						media_codec->description);
+				break;
+			case SPA_BT_PROFILE_BAP_SOURCE:
+				desc_and_codec = spa_aprintf(_("High Fidelity Input (BAP Source, codec %s)"),
+						media_codec->description);
+				break;
+			default:
+				desc_and_codec = spa_aprintf(_("High Fidelity Duplex (BAP Source/Sink, codec %s)"),
+						media_codec->description);
+			}
+			desc = desc_and_codec;
+			priority = 128 + this->supported_codec_count - idx;  /* order as in codec list */
+		} else {
+			switch (profile) {
+			case SPA_BT_PROFILE_BAP_SINK:
+				desc = _("High Fidelity Playback (BAP Sink)");
+				break;
+			case SPA_BT_PROFILE_BAP_SOURCE:
+				desc = _("High Fidelity Input (BAP Source)");
+				break;
+			default:
+				desc = _("High Fidelity Duplex (BAP Source/Sink)");
+			}
+			priority = 128;
 		}
-		desc = desc_and_codec;
-		priority = 128 + this->supported_codec_count - idx;  /* order as in codec list */
 		break;
 	}
 	case DEVICE_PROFILE_HSP_HFP:
@@ -2107,7 +2166,7 @@ static int impl_set_param(void *object,
 				SPA_PARAM_PROFILE_index, SPA_POD_Int(&idx),
 				SPA_PARAM_PROFILE_save, SPA_POD_OPT_Bool(&save))) < 0) {
 			spa_log_warn(this->log, "can't parse profile");
-			spa_debug_pod(0, NULL, param);
+			spa_debug_log_pod(this->log, SPA_LOG_LEVEL_DEBUG, 0, NULL, param);
 			return res;
 		}
 
@@ -2135,7 +2194,7 @@ static int impl_set_param(void *object,
 				SPA_PARAM_ROUTE_props, SPA_POD_OPT_Pod(&props),
 				SPA_PARAM_ROUTE_save, SPA_POD_OPT_Bool(&save))) < 0) {
 			spa_log_warn(this->log, "can't parse route");
-			spa_debug_pod(0, NULL, param);
+			spa_debug_log_pod(this->log, SPA_LOG_LEVEL_DEBUG, 0, NULL, param);
 			return res;
 		}
 		if (device > 1 || !this->nodes[device].active)
@@ -2166,7 +2225,7 @@ static int impl_set_param(void *object,
 				SPA_PROP_bluetoothAudioCodec, SPA_POD_OPT_Id(&codec_id),
 				SPA_PROP_bluetoothOffloadActive, SPA_POD_OPT_Bool(&offload_active))) < 0) {
 			spa_log_warn(this->log, "can't parse props");
-			spa_debug_pod(0, NULL, param);
+			spa_debug_log_pod(this->log, SPA_LOG_LEVEL_DEBUG, 0, NULL, param);
 			return res;
 		}
 
