@@ -35,6 +35,7 @@
 #include <limits.h>
 
 #include <linux/media.h>
+#include <libcamera/control_ids.h>
 
 int spa_libcamera_open(struct impl *impl)
 {
@@ -74,7 +75,7 @@ static void spa_libcamera_get_config(struct impl *impl)
 		return;
 
 	StreamRoles roles;
-	roles.push_back(VideoRecording);
+	roles.push_back(StreamRole::VideoRecording);
 	impl->config = impl->camera->generateConfiguration(roles);
 }
 
@@ -458,6 +459,38 @@ error:
 
 }
 
+static struct {
+	uint32_t id;
+	uint32_t spa_id;
+} control_map[] = {
+	{ libcamera::controls::BRIGHTNESS, SPA_PROP_brightness },
+	{ libcamera::controls::CONTRAST, SPA_PROP_contrast },
+	{ libcamera::controls::SATURATION, SPA_PROP_saturation },
+	{ libcamera::controls::EXPOSURE_TIME, SPA_PROP_exposure },
+	{ libcamera::controls::ANALOGUE_GAIN, SPA_PROP_gain },
+	{ libcamera::controls::SHARPNESS, SPA_PROP_sharpness },
+};
+
+static uint32_t control_to_prop_id(struct impl *impl, uint32_t control_id)
+{
+	SPA_FOR_EACH_ELEMENT_VAR(control_map, c) {
+		if (c->id == control_id)
+			return c->spa_id;
+	}
+	return SPA_PROP_START_CUSTOM + control_id;
+}
+
+static uint32_t prop_id_to_control(struct impl *impl, uint32_t prop_id)
+{
+	SPA_FOR_EACH_ELEMENT_VAR(control_map, c) {
+		if (c->spa_id == prop_id)
+			return c->id;
+	}
+	if (prop_id >= SPA_PROP_START_CUSTOM)
+		return prop_id - SPA_PROP_START_CUSTOM;
+	return SPA_ID_INVALID;
+}
+
 static int
 spa_libcamera_enum_controls(struct impl *impl, struct port *port, int seq,
 		       uint32_t start, uint32_t num,
@@ -469,7 +502,7 @@ spa_libcamera_enum_controls(struct impl *impl, struct port *port, int seq,
 	struct spa_pod_frame f[2];
 	struct spa_result_node_params result;
 	struct spa_pod *ctrl;
-	uint32_t count = 0, skip;
+	uint32_t count = 0, skip, id;
 	int res;
 	const ControlId *ctrl_id;
 	ControlInfo ctrl_info;
@@ -492,36 +525,58 @@ next:
 	ctrl_id = it->first;
 	ctrl_info = it->second;
 
+	id = control_to_prop_id(impl, ctrl_id->id());
+
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
 	spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_PropInfo, SPA_PARAM_PropInfo);
 	spa_pod_builder_add(&b,
-			SPA_PROP_INFO_id,   SPA_POD_Id(ctrl_id->id()),
+			SPA_PROP_INFO_id,   SPA_POD_Id(id),
 			SPA_PROP_INFO_description, SPA_POD_String(ctrl_id->name().c_str()),
 			0);
 
 	switch (ctrl_id->type()) {
-	case ControlTypeBool:
+	case ControlTypeBool: {
+		bool def;
+		if (ctrl_info.def().isNone())
+			def = ctrl_info.min().get<bool>();
+		else
+			def = ctrl_info.def().get<bool>();
+
 		spa_pod_builder_add(&b,
-				SPA_PROP_INFO_type, SPA_POD_CHOICE_Bool(
-						(bool)ctrl_info.def().get<bool>()),
-				0);
-		break;
-	case ControlTypeFloat:
+					SPA_PROP_INFO_type, SPA_POD_CHOICE_Bool(
+							def),
+					0);
+	} break;
+	case ControlTypeFloat: {
+		float min = ctrl_info.min().get<float>();
+		float max = ctrl_info.max().get<float>();
+		float def;
+
+		if (ctrl_info.def().isNone())
+			def = (min + max) / 2;
+		else
+			def = ctrl_info.def().get<float>();
+
 		spa_pod_builder_add(&b,
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Float(
-						(float)ctrl_info.def().get<float>(),
-						(float)ctrl_info.min().get<float>(),
-						(float)ctrl_info.max().get<float>()),
+						def, min, max),
 				0);
-		break;
-	case ControlTypeInteger32:
+	} break;
+	case ControlTypeInteger32: {
+		int32_t min = ctrl_info.min().get<int32_t>();
+		int32_t max = ctrl_info.max().get<int32_t>();
+		int32_t def;
+
+		if (ctrl_info.def().isNone())
+			def = (min + max) / 2;
+		else
+			def = ctrl_info.def().get<int32_t>();
+
 		spa_pod_builder_add(&b,
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Int(
-						(int32_t)ctrl_info.def().get<int32_t>(),
-						(int32_t)ctrl_info.min().get<int32_t>(),
-						(int32_t)ctrl_info.max().get<int32_t>()),
+						def, min, max),
 				0);
-		break;
+	} break;
 	default:
 		goto next;
 	}
@@ -581,8 +636,13 @@ spa_libcamera_set_control(struct impl *impl, const struct spa_pod_prop *prop)
 	const ControlId *ctrl_id;
 	int res;
 	struct val d;
+	uint32_t control_id;
 
-	auto v = info.idmap().find(prop->key);
+	control_id = prop_id_to_control(impl, prop->key);
+	if (control_id == SPA_ID_INVALID)
+		return -ENOENT;
+
+	auto v = info.idmap().find(control_id);
 	if (v == info.idmap().end())
 		return -ENOENT;
 
@@ -678,6 +738,31 @@ static int spa_libcamera_use_buffers(struct impl *impl, struct port *port,
 	return -ENOTSUP;
 }
 
+static const struct {
+	Transform libcamera_transform;
+	uint32_t spa_transform_value;
+} transform_map[] = {
+	{ Transform::Identity, SPA_META_TRANSFORMATION_None },
+	{ Transform::Rot0, SPA_META_TRANSFORMATION_None },
+	{ Transform::HFlip, SPA_META_TRANSFORMATION_Flipped },
+	{ Transform::VFlip, SPA_META_TRANSFORMATION_Flipped180 },
+	{ Transform::HVFlip, SPA_META_TRANSFORMATION_180 },
+	{ Transform::Rot180, SPA_META_TRANSFORMATION_180 },
+	{ Transform::Transpose, SPA_META_TRANSFORMATION_Flipped90 },
+	{ Transform::Rot90, SPA_META_TRANSFORMATION_90 },
+	{ Transform::Rot270, SPA_META_TRANSFORMATION_270 },
+	{ Transform::Rot180Transpose, SPA_META_TRANSFORMATION_Flipped270 },
+};
+
+static uint32_t libcamera_transform_to_spa_transform_value(Transform transform)
+{
+	for (const auto& t : transform_map) {
+		if (t.libcamera_transform == transform)
+			return t.spa_transform_value;
+	}
+	return SPA_META_TRANSFORMATION_None;
+}
+
 static int
 mmap_init(struct impl *impl, struct port *port,
 		struct spa_buffer **buffers, uint32_t n_buffers)
@@ -722,6 +807,16 @@ mmap_init(struct impl *impl, struct port *port,
 		b->flags = BUFFER_FLAG_OUTSTANDING;
 		b->h = (struct spa_meta_header*)spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
+		b->videotransform = (struct spa_meta_videotransform*)spa_buffer_find_meta_data(
+			buffers[i], SPA_META_VideoTransform, sizeof(*b->videotransform));
+		if (b->videotransform) {
+			b->videotransform->transform =
+				libcamera_transform_to_spa_transform_value(impl->config->transform);
+			spa_log_debug(impl->log, "Setting videotransform for buffer %d to %u (from %s)",
+				i, b->videotransform->transform, transformToString(impl->config->transform));
+
+		}
+
 		d = buffers[i]->datas;
 		for(j = 0; j < buffers[i]->n_datas; ++j) {
 			d[j].type = port->memtype;
