@@ -27,6 +27,8 @@
 
 #include <gst/gst.h>
 
+#include <gst/video/video.h>
+
 #include <pipewire/pipewire.h>
 
 G_BEGIN_DECLS
@@ -53,9 +55,11 @@ struct _GstPipeWirePoolData {
   void *owner;
   struct spa_meta_header *header;
   guint flags;
-  goffset offset;
   struct pw_buffer *b;
   GstBuffer *buf;
+  gboolean queued;
+  struct spa_meta_region *crop;
+  struct spa_meta_videotransform *videotransform;
 };
 
 struct _GstPipeWirePool {
@@ -64,6 +68,9 @@ struct _GstPipeWirePool {
   struct pw_stream *stream;
   struct pw_type *t;
 
+  gboolean add_metavideo;
+  GstVideoInfo video_info;
+
   GstAllocator *fd_allocator;
   GstAllocator *dmabuf_allocator;
 
