@@ -142,6 +142,7 @@ gst_pipewire_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
   GstPipeWireSink *pwsink = GST_PIPEWIRE_SINK (bsink);
 
   gst_query_add_allocation_pool (query, GST_BUFFER_POOL_CAST (pwsink->pool), 0, 0, 0);
+  gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
   return TRUE;
 }
 
@@ -346,6 +347,14 @@ gst_pipewire_sink_sink_fixate (GstBaseSink * bsink, GstCaps * caps)
     gst_structure_fixate_field_string (structure, "format", "S16LE");
     gst_structure_fixate_field_nearest_int (structure, "channels", 2);
     gst_structure_fixate_field_nearest_int (structure, "rate", 44100);
+  } else if (gst_structure_has_name (structure, "audio/mpeg")) {
+    gst_structure_fixate_field_string (structure, "format", "Encoded");
+    gst_structure_fixate_field_nearest_int (structure, "channels", 2);
+    gst_structure_fixate_field_nearest_int (structure, "rate", 44100);
+  } else if (gst_structure_has_name (structure, "audio/x-flac")) {
+    gst_structure_fixate_field_string (structure, "format", "Encoded");
+    gst_structure_fixate_field_nearest_int (structure, "channels", 2);
+    gst_structure_fixate_field_nearest_int (structure, "rate", 44100);
   }
 
   caps = GST_BASE_SINK_CLASS (parent_class)->fixate (bsink, caps);
@@ -496,6 +505,22 @@ do_send_buffer (GstPipeWireSink *pwsink, GstBuffer *buffer)
     d->chunk->stride = 0;
   }
 
+  GstVideoMeta *meta = gst_buffer_get_video_meta (buffer);
+  if (meta) {
+    if (meta->n_planes == b->n_datas) {
+      gsize video_size = 0;
+      for (i = 0; i < meta->n_planes; i++) {
+        struct spa_data *d = &b->datas[i];
+        d->chunk->offset += meta->offset[i] - video_size;
+        d->chunk->stride = meta->stride[i];
+
+        video_size += d->chunk->size;
+      }
+    } else {
+      GST_ERROR ("plane num not matching, meta:%u buffer:%u", meta->n_planes, b->n_datas);
+    }
+  }
+
   if ((res = pw_stream_queue_buffer (pwsink->stream, data->b)) < 0) {
     g_warning ("can't send buffer %s", spa_strerror(res));
   }
