@@ -428,6 +428,9 @@ gst_pipewire_src_init (GstPipeWireSrc * src)
   /* we operate in time */
   gst_base_src_set_format (GST_BASE_SRC (src), GST_FORMAT_TIME);
 
+  /* we're a live source, unless explicitly requested not to be */
+  gst_base_src_set_live (GST_BASE_SRC (src), TRUE);
+
   GST_OBJECT_FLAG_SET (src, GST_ELEMENT_FLAG_PROVIDE_CLOCK);
 
   src->always_copy = DEFAULT_ALWAYS_COPY;
@@ -612,6 +615,27 @@ static GstBuffer *dequeue_buffer(GstPipeWireSrc *pwsrc)
     }
   }
 
+  if (pwsrc->is_video) {
+    gsize video_size = 0;
+    GstVideoInfo *info = &pwsrc->video_info;
+    GstVideoMeta *meta = gst_buffer_add_video_meta_full (buf, GST_VIDEO_FRAME_FLAG_NONE,
+                             GST_VIDEO_INFO_FORMAT (info),
+                             GST_VIDEO_INFO_WIDTH (info),
+                             GST_VIDEO_INFO_HEIGHT (info),
+                             GST_VIDEO_INFO_N_PLANES (info),
+                             info->offset,
+                             info->stride);
+
+    meta->n_planes = MIN(meta->n_planes, b->buffer->n_datas);
+    for (i = 0; i < meta->n_planes; i++) {
+      struct spa_data *d = &b->buffer->datas[i];
+      meta->offset[i] = video_size;
+      meta->stride[i] = d->chunk->stride;
+
+      video_size += d->chunk->size;
+    }
+  }
+
   for (i = 0; i < b->buffer->n_datas; i++) {
     struct spa_data *d = &b->buffer->datas[i];
     GstMemory *pmem = gst_buffer_peek_memory (data->buf, i);
@@ -655,6 +679,7 @@ on_state_changed (void *data,
     case PW_STREAM_STATE_STREAMING:
       break;
     case PW_STREAM_STATE_ERROR:
+      pw_stream_set_error (pwsrc->stream, -EPIPE, "%s", error);
       GST_ELEMENT_ERROR (pwsrc, RESOURCE, FAILED,
           ("stream error: %s", error), (NULL));
       break;
@@ -670,7 +695,7 @@ parse_stream_properties (GstPipeWireSrc *pwsrc, const struct pw_properties *prop
 
   GST_OBJECT_LOCK (pwsrc);
   var = pw_properties_get (props, PW_KEY_STREAM_IS_LIVE);
-  is_live = pwsrc->is_live = var ? pw_properties_parse_bool(var) : FALSE;
+  is_live = pwsrc->is_live = var ? pw_properties_parse_bool(var) : TRUE;
 
   var = pw_properties_get (props, PW_KEY_STREAM_LATENCY_MIN);
   pwsrc->min_latency = var ? (GstClockTime) atoi (var) : 0;
@@ -955,6 +980,10 @@ on_param_changed (void *data, uint32_t id,
           gst_caps_unref(pwsrc->caps);
   pwsrc->caps = gst_caps_from_format (param);
 
+  pwsrc->is_video = pwsrc->caps != NULL
+                      ? gst_video_info_from_caps (&pwsrc->video_info, pwsrc->caps)
+                      : FALSE;
+
   pwsrc->negotiated = pwsrc->caps != NULL;
 
   if (pwsrc->negotiated) {
@@ -964,14 +993,14 @@ on_param_changed (void *data, uint32_t id,
     uint32_t buffers = CLAMP (16, pwsrc->min_buffers, pwsrc->max_buffers);
     int buffertypes;
 
+    buffertypes = (1<<SPA_DATA_DmaBuf);
     if (spa_pod_find_prop (param, NULL, SPA_FORMAT_VIDEO_modifier)) {
-      buffertypes = (1<<SPA_DATA_DmaBuf);
       gst_caps_features_remove (gst_caps_get_features (pwsrc->caps, 0),
           GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
       gst_caps_features_add (gst_caps_get_features (pwsrc->caps, 0),
           GST_CAPS_FEATURE_MEMORY_DMABUF);
     } else {
-      buffertypes = ((1<<SPA_DATA_MemFd) | (1<<SPA_DATA_MemPtr));
+      buffertypes |= ((1<<SPA_DATA_MemFd) | (1<<SPA_DATA_MemPtr));
     }
 
     GST_DEBUG_OBJECT (pwsrc, "we got format %" GST_PTR_FORMAT, pwsrc->caps);
