@@ -28,6 +28,8 @@
 #include <gst/gst.h>
 #include <gst/base/gstpushsrc.h>
 
+#include <gst/video/video.h>
+
 #include <pipewire/pipewire.h>
 #include <gst/gstpipewirepool.h>
 #include <gst/gstpipewirecore.h>
@@ -71,6 +73,9 @@ struct _GstPipeWireSrc {
 
   GstCaps *caps;
 
+  gboolean is_video;
+  GstVideoInfo video_info;
+
   gboolean negotiated;
   gboolean flushing;
   gboolean started;
@@ -80,6 +85,7 @@ struct _GstPipeWireSrc {
   GstClockTime min_latency;
   GstClockTime max_latency;
 
+  GstStructure *client_properties;
   GstPipeWireCore *core;
   struct spa_hook core_listener;
   int last_seq;
@@ -89,11 +95,13 @@ struct _GstPipeWireSrc {
   struct spa_hook stream_listener;
 
   GstBuffer *last_buffer;
-  GstStructure *properties;
+  GstStructure *stream_properties;
 
   GstPipeWirePool *pool;
   GstClock *clock;
   GstClockTime last_time;
+
+  enum spa_meta_videotransform_value transform_value;
 };
 
 struct _GstPipeWireSrcClass {
