@@ -22,6 +22,9 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#ifndef PLUGIN_H
+#define PLUGIN_H
+
 #include <stdint.h>
 #include <stddef.h>
 #include <errno.h>
@@ -33,6 +36,8 @@
 #include <spa/utils/list.h>
 #include <spa/utils/string.h>
 
+#include "dsp-ops.h"
+
 struct fc_plugin {
 	const struct fc_descriptor *(*make_desc)(struct fc_plugin *plugin, const char *name);
 	void (*unload) (struct fc_plugin *plugin);
@@ -78,6 +83,7 @@ struct fc_descriptor {
 	void (*cleanup) (void *instance);
 
 	void (*connect_port) (void *instance, unsigned long port, float *data);
+	void (*control_changed) (void *instance);
 
 	void (*activate) (void *instance);
 	void (*deactivate) (void *instance);
@@ -98,8 +104,12 @@ static inline void fc_descriptor_free(const struct fc_descriptor *desc)
 }
 
 struct fc_plugin *load_ladspa_plugin(const struct spa_support *support, uint32_t n_support,
-		const char *path, const char *config);
+		struct dsp_ops *dsp, const char *path, const char *config);
 struct fc_plugin *load_lv2_plugin(const struct spa_support *support, uint32_t n_support,
-		const char *path, const char *config);
+		struct dsp_ops *dsp, const char *path, const char *config);
 struct fc_plugin *load_builtin_plugin(const struct spa_support *support, uint32_t n_support,
-		const char *path, const char *config);
+		struct dsp_ops *dsp, const char *path, const char *config);
+struct fc_plugin *load_sofa_plugin(const struct spa_support *support, uint32_t n_support,
+		struct dsp_ops *dsp, const char *path, const char *config);
+
+#endif /* PLUGIN_H */
