@@ -25,6 +25,7 @@
 #include "config.h"
 
 #include <limits.h>
+#include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -34,14 +35,15 @@
 #include <net/if.h>
 #include <ctype.h>
 
-#include <spa/param/audio/format-utils.h>
 #include <spa/utils/hook.h>
+#include <spa/utils/result.h>
 #include <spa/utils/ringbuffer.h>
 #include <spa/utils/dll.h>
+#include <spa/param/audio/format-utils.h>
 #include <spa/debug/mem.h>
 
 #include <pipewire/pipewire.h>
-#include <pipewire/private.h>
+#include <pipewire/impl.h>
 
 #include <module-rtp/sap.h>
 #include <module-rtp/rtp.h>
@@ -77,19 +79,42 @@
  * ## Example configuration
  *\code{.unparsed}
  * context.modules = [
- *  {   name = libpipewire-module-rtp-source
- *      args = {
- *          #sap.ip = 224.0.0.56
- *          #sap.port = 9875
- *          #local.ifname = eth0
- *          sess.latency.msec = 100
- *          stream.props = {
- *             node.name = "rtp-source"
- *             #media.class = "Audio/Source"
- *          }
- *      }
- *  }
- *]
+ * {   name = libpipewire-module-rtp-source
+ *     args = {
+ *         #sap.ip = 224.0.0.56
+ *         #sap.port = 9875
+ *         #local.ifname = eth0
+ *         sess.latency.msec = 100
+ *         #node.always-process = false # true to receive even when not running
+ *         stream.props = {
+ *            #media.class = "Audio/Source"
+ *            #node.name = "rtp-source"
+ *         }
+ *         stream.rules = [
+ *             {   matches = [
+ *                     # any of the items in matches needs to match, if one does,
+ *                     # actions are emited.
+ *                     {   # all keys must match the value. ~ in value starts regex.
+ *                         #rtp.origin = "wim 3883629975 0 IN IP4 0.0.0.0"
+ *                         #rtp.payload = "127"
+ *                         #rtp.fmt = "L16/48000/2"
+ *                         #rtp.session = "PipeWire RTP Stream on fedora"
+ *                         #rtp.ts-offset = 0
+ *                         #rtp.ts-refclk = "private"
+ *                     }
+ *                 ]
+ *                 actions = {
+ *                     create-stream = {
+ *                         #sess.latency.msec = 100
+ *                         #sess.ts-direct = false
+ *                         #target.object = ""
+ *                     }
+ *                 }
+ *             }
+ *         ]
+ *     }
+ * }
+ * ]
  *\endcode
  *
  * \since 0.3.60
@@ -100,24 +125,25 @@
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 #define PW_LOG_TOPIC_DEFAULT mod_topic
 
-#define SAP_MIME_TYPE		"application/sdp"
+#define SAP_MIME_TYPE			"application/sdp"
 
-#define ERROR_MSEC		2
-#define MAX_SESSIONS		16
-#define CLEANUP_INTERVAL_SEC	20
+#define ERROR_MSEC			2
+#define MAX_SESSIONS			16
 
-#define DEFAULT_SAP_IP		"224.0.0.56"
-#define DEFAULT_SAP_PORT	9875
-#define DEFAULT_SESS_LATENCY	100
+#define DEFAULT_CLEANUP_INTERVAL_SEC	90
+#define DEFAULT_SAP_IP			"224.0.0.56"
+#define DEFAULT_SAP_PORT		9875
+#define DEFAULT_SESS_LATENCY		100
 
-#define BUFFER_SIZE		(1u<<22)
-#define BUFFER_MASK		(BUFFER_SIZE-1)
+#define BUFFER_SIZE			(1u<<22)
+#define BUFFER_MASK			(BUFFER_SIZE-1)
 
 #define USAGE	"sap.ip=<SAP IP address to listen on, default "DEFAULT_SAP_IP"> "				\
 		"sap.port=<SAP port to listen on, default "SPA_STRINGIFY(DEFAULT_SAP_PORT)"> "			\
 		"local.ifname=<local interface name to use> "							\
 		"sess.latency.msec=<target network latency, default "SPA_STRINGIFY(DEFAULT_SESS_LATENCY)"> "	\
-		"stream.props= { key=value ... }"
+		"stream.props= { key=value ... } "								\
+		"stream.rules=<rules> "
 
 static const struct spa_dict_item module_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Wim Taymans <wim.taymans@gmail.com>" },
@@ -148,8 +174,10 @@ struct impl {
 
 	char *ifname;
 	char *sap_ip;
+	bool always_process;
 	int sap_port;
 	int sess_latency_msec;
+	uint32_t cleanup_interval;
 
 	struct spa_list sessions;
 	uint32_t n_sessions;
@@ -180,6 +208,7 @@ struct sdp_info {
 
 	char origin[128];
 	char session[256];
+	char channelmap[512];
 
 	struct sockaddr_storage sa;
 	socklen_t salen;
@@ -190,6 +219,9 @@ struct sdp_info {
 	const struct format_info *format_info;
 	struct spa_audio_info_raw info;
 	uint32_t stride;
+
+	uint32_t ts_offset;
+	char refclk[64];
 };
 
 struct session {
@@ -215,10 +247,13 @@ struct session {
 	uint8_t buffer[BUFFER_SIZE];
 
 	struct spa_io_rate_match *rate_match;
+	struct spa_io_position *position;
 	struct spa_dll dll;
 	uint32_t target_buffer;
 	float max_error;
-	unsigned buffering:1;
+	unsigned first:1;
+	unsigned receiving:1;
+	unsigned direct_timestamp:1;
 };
 
 static void stream_destroy(void *d)
@@ -233,8 +268,8 @@ static void stream_process(void *data)
 	struct session *sess = data;
 	struct pw_buffer *buf;
 	struct spa_data *d;
-	uint32_t index;
-        int32_t avail, wanted;
+	uint32_t wanted, timestamp, target_buffer, stride, maxsize;
+	int32_t avail;
 
 	if ((buf = pw_stream_dequeue_buffer(sess->stream)) == NULL) {
 		pw_log_debug("Out of stream buffers: %m");
@@ -242,75 +277,84 @@ static void stream_process(void *data)
 	}
 	d = buf->buffer->datas;
 
-	wanted = buf->requested ?
-		SPA_MIN(buf->requested * sess->info.stride, d[0].maxsize)
-		: d[0].maxsize;
+	stride = sess->info.stride;
 
-	avail = spa_ringbuffer_get_read_index(&sess->ring, &index);
+	maxsize = d[0].maxsize / stride;
+	wanted = buf->requested ? SPA_MIN(buf->requested, maxsize) : maxsize;
 
-        if (avail < wanted || sess->buffering) {
-                memset(d[0].data, 0, wanted);
-		if (!sess->buffering && sess->have_sync) {
-			pw_log_info("underrun %u/%u < %u, buffering...",
-					avail, sess->target_buffer, wanted);
-			sess->buffering = true;
-		}
-        } else {
-		float error, corr;
-		if (avail > (int32_t)BUFFER_SIZE) {
-			index += avail - sess->target_buffer;
-			avail = sess->target_buffer;
-			pw_log_warn("overrun %u > %u", avail, BUFFER_SIZE);
+	if (sess->position && sess->direct_timestamp) {
+		/* in direct mode, read directly from the timestamp index,
+		 * because sender and receiver are in sync, this would keep
+		 * target_buffer of bytes available. */
+		spa_ringbuffer_read_update(&sess->ring,
+				sess->position->clock.position);
+	}
+	avail = spa_ringbuffer_get_read_index(&sess->ring, &timestamp);
+
+	target_buffer = sess->target_buffer;
+
+	if (avail < (int32_t)wanted) {
+		enum spa_log_level level;
+		memset(d[0].data, 0, wanted * stride);
+		if (sess->have_sync) {
+			sess->have_sync = false;
+			level = SPA_LOG_LEVEL_WARN;
 		} else {
-			error = (float)sess->target_buffer - (float)avail;
+			level = SPA_LOG_LEVEL_DEBUG;
+		}
+		pw_log(level, "underrun %d/%u < %u",
+					avail, target_buffer, wanted);
+	} else {
+		float error, corr;
+		if (avail > (int32_t)SPA_MIN(target_buffer * 8, BUFFER_SIZE / stride)) {
+			pw_log_warn("overrun %u > %u", avail, target_buffer * 8);
+			timestamp += avail - target_buffer;
+			avail = target_buffer;
+		} else if (sess->first) {
+			if ((uint32_t)avail > target_buffer) {
+				uint32_t skip = avail - target_buffer;
+				pw_log_debug("first: avail:%d skip:%u target:%u",
+							avail, skip, target_buffer);
+				timestamp += skip;
+				avail = target_buffer;
+			}
+			sess->first = false;
+		}
+		if (!sess->direct_timestamp) {
+			/* when not using direct timestamp and clocks are not
+			 * in sync, try to adjust our playback rate to keep the
+			 * requested target_buffer bytes in the ringbuffer */
+			error = (float)target_buffer - (float)avail;
 			error = SPA_CLAMP(error, -sess->max_error, sess->max_error);
 
 			corr = spa_dll_update(&sess->dll, error);
 
 			pw_log_debug("avail:%u target:%u error:%f corr:%f", avail,
-					sess->target_buffer, error, corr);
+					target_buffer, error, corr);
 
 			if (sess->rate_match) {
-				SPA_FLAG_SET(sess->rate_match->flags, SPA_IO_RATE_MATCH_FLAG_ACTIVE);
+				SPA_FLAG_SET(sess->rate_match->flags,
+						SPA_IO_RATE_MATCH_FLAG_ACTIVE);
 				sess->rate_match->rate = 1.0f / corr;
 			}
 		}
 		spa_ringbuffer_read_data(&sess->ring,
 				sess->buffer,
 				BUFFER_SIZE,
-                                index & BUFFER_MASK,
-                                d[0].data, wanted);
+				(timestamp * stride) & BUFFER_MASK,
+				d[0].data, wanted * stride);
 
-                index += wanted;
-                spa_ringbuffer_read_update(&sess->ring, index);
-        }
-        d[0].chunk->size = wanted;
-        d[0].chunk->stride = sess->info.stride;
-        d[0].chunk->offset = 0;
-        buf->size = wanted / sess->info.stride;
+		timestamp += wanted;
+		spa_ringbuffer_read_update(&sess->ring, timestamp);
+	}
+	d[0].chunk->size = wanted * stride;
+	d[0].chunk->stride = stride;
+	d[0].chunk->offset = 0;
+	buf->size = wanted;
 
 	pw_stream_queue_buffer(sess->stream, buf);
 }
 
-static void on_stream_state_changed(void *d, enum pw_stream_state old,
-		enum pw_stream_state state, const char *error)
-{
-	struct session *sess = d;
-	struct impl *impl = sess->impl;
-
-	switch (state) {
-	case PW_STREAM_STATE_UNCONNECTED:
-		pw_log_info("stream disconnected, unloading");
-		pw_impl_module_schedule_destroy(impl->module);
-		break;
-	case PW_STREAM_STATE_ERROR:
-		pw_log_error("stream error: %s", error);
-		break;
-	default:
-		break;
-	}
-}
-
 static void stream_io_changed(void *data, uint32_t id, void *area, uint32_t size)
 {
 	struct session *sess = data;
@@ -318,16 +362,18 @@ static void stream_io_changed(void *data, uint32_t id, void *area, uint32_t size
 	case SPA_IO_RateMatch:
 		sess->rate_match = area;
 		break;
+	case SPA_IO_Position:
+		sess->position = area;
+		break;
 	}
 }
 
-static const struct pw_stream_events out_stream_events = {
-	PW_VERSION_STREAM_EVENTS,
-	.destroy = stream_destroy,
-	.state_changed = on_stream_state_changed,
-	.io_changed = stream_io_changed,
-	.process = stream_process
-};
+static void session_touch(struct session *sess)
+{
+	struct timespec ts;
+	clock_gettime(CLOCK_MONOTONIC, &ts);
+	sess->timestamp = SPA_TIMESPEC_TO_NSEC(&ts);
+}
 
 static void
 on_rtp_io(void *data, int fd, uint32_t mask)
@@ -338,11 +384,10 @@ on_rtp_io(void *data, int fd, uint32_t mask)
 	uint8_t buffer[2048], *payload;
 
 	if (mask & SPA_IO_IN) {
-		uint32_t index, expected_index, timestamp;
+		uint32_t stride, read, timestamp, expected_timestamp, samples;
 		uint16_t seq;
 		int32_t filled;
 
-		pw_log_trace("got rtp");
 		if ((len = recv(fd, buffer, sizeof(buffer), 0)) < 0)
 			goto receive_error;
 
@@ -364,57 +409,54 @@ on_rtp_io(void *data, int fd, uint32_t mask)
 
 		seq = ntohs(hdr->sequence_number);
 		if (sess->have_seq && sess->expected_seq != seq) {
-			pw_log_warn("unexpected seq (%d != %d)", seq, sess->expected_seq);
+			pw_log_info("unexpected seq (%d != %d)", seq, sess->expected_seq);
+			sess->have_sync = false;
 		}
 		sess->expected_seq = seq + 1;
 		sess->have_seq = true;
 
-		len = SPA_ROUND_DOWN(len - hlen, sess->info.stride);
+		stride = sess->info.stride;
+		samples = (len - hlen) / stride;
 		payload = &buffer[hlen];
 
-		filled = spa_ringbuffer_get_write_index(&sess->ring, &index);
+		filled = spa_ringbuffer_get_write_index(&sess->ring, &expected_timestamp);
 
-		timestamp = ntohl(hdr->timestamp);
-		expected_index = timestamp * sess->info.stride;
+		read = ntohl(hdr->timestamp) - sess->info.ts_offset;
+		/* we always write to timestamp + delay */
+		timestamp = read + sess->target_buffer;
 
 		if (!sess->have_sync) {
-			sess->ring.readindex = sess->ring.writeindex =
-				index = expected_index;
-			filled = 0;
-			sess->have_sync = true;
-			sess->buffering = true;
-			pw_log_info("sync to timestamp %u", index);
+			pw_log_info("sync to timestamp %u", read);
+			/* we read from timestamp, keeping target_buffer of data
+			 * in the ringbuffer. */
+			sess->ring.readindex = read;
+			sess->ring.writeindex = timestamp;
+			filled = sess->target_buffer;
 
 			spa_dll_init(&sess->dll);
 			spa_dll_set_bw(&sess->dll, SPA_DLL_BW_MIN, 128, sess->info.info.rate);
-
-		} else if (expected_index != index) {
+			memset(sess->buffer, 0, BUFFER_SIZE);
+			sess->have_sync = true;
+		} else if (expected_timestamp != timestamp) {
 			pw_log_debug("unexpected timestamp (%u != %u)",
-					index / sess->info.stride,
-					expected_index / sess->info.stride);
-			index = expected_index;
-			filled = 0;
+					timestamp, expected_timestamp);
 		}
 
-		if (filled + len > BUFFER_SIZE) {
-			pw_log_warn("capture overrun %u %zd", filled, len);
+		if (filled + samples > BUFFER_SIZE / stride) {
+			pw_log_debug("capture overrun %u + %u > %u", filled, samples,
+					BUFFER_SIZE / stride);
 			sess->have_sync = false;
 		} else {
+			pw_log_trace("got samples:%u", samples);
 			spa_ringbuffer_write_data(&sess->ring,
 					sess->buffer,
 					BUFFER_SIZE,
-					index & BUFFER_MASK,
-					payload, len);
-			index += len;
-			filled += len;
-			spa_ringbuffer_write_update(&sess->ring, index);
-
-			if (sess->buffering && (uint32_t)filled > sess->target_buffer) {
-				sess->buffering = false;
-				pw_log_info("buffering done %u > %u",
-					filled, sess->target_buffer);
-			}
+					(timestamp * stride) & BUFFER_MASK,
+					payload, (samples * stride));
+			timestamp += samples;
+			spa_ringbuffer_write_update(&sess->ring, timestamp);
 		}
+		sess->receiving = true;
 	}
 	return;
 
@@ -513,22 +555,15 @@ error:
 	return res;
 }
 
-static uint32_t msec_to_bytes(struct sdp_info *info, uint32_t msec)
+static uint32_t msec_to_samples(struct sdp_info *info, uint32_t msec)
 {
-	return msec * info->stride * info->info.rate / 1000;
-}
-
-static void session_touch(struct session *sess)
-{
-	struct timespec ts;
-	clock_gettime(CLOCK_MONOTONIC, &ts);
-	sess->timestamp = SPA_TIMESPEC_TO_NSEC(&ts);
+	return msec * info->info.rate / 1000;
 }
 
 static void session_free(struct session *sess)
 {
-	pw_log_info("free session %s %s", sess->info.origin, sess->info.session);
 	if (sess->impl) {
+		pw_log_info("free session %s %s", sess->info.origin, sess->info.session);
 		sess->impl->n_sessions--;
 		spa_list_remove(&sess->link);
 	}
@@ -539,6 +574,97 @@ static void session_free(struct session *sess)
 	free(sess);
 }
 
+struct session_info {
+	struct session *session;
+	struct pw_properties *props;
+	bool matched;
+};
+
+static int rule_matched(void *data, const char *location, const char *action,
+                        const char *str, size_t len)
+{
+	struct session_info *i = data;
+	int res = 0;
+
+	i->matched = true;
+	if (spa_streq(action, "create-stream")) {
+		pw_properties_update_string(i->props, str, len);
+	}
+	return res;
+}
+
+static int session_start(struct impl *impl, struct session *session) {
+	int fd;
+	if (session->source)
+	  return 0;
+
+	pw_log_info("starting RTP listener");
+
+	if ((fd = make_socket((const struct sockaddr *)&session->info.sa,
+					session->info.salen, impl->ifname)) < 0) {
+		pw_log_error("failed to create socket: %m");
+		return fd;
+	}
+
+	session->source = pw_loop_add_io(impl->data_loop, fd,
+				SPA_IO_IN, true, on_rtp_io, session);
+	if (session->source == NULL) {
+		pw_log_error("can't create io source: %m");
+		close(fd);
+		return -errno;
+	}
+	return 0;
+}
+
+static void session_stop(struct impl *impl, struct session *session) {
+	if (!session->source)
+		return;
+
+	pw_log_info("stopping RTP listener");
+
+	pw_loop_destroy_source(
+		session->impl->data_loop,
+		session->source
+	);
+
+	session->source = NULL;
+}
+
+static void on_stream_state_changed(void *d, enum pw_stream_state old,
+		enum pw_stream_state state, const char *error)
+{
+	struct session *sess = d;
+	struct impl *impl = sess->impl;
+
+	switch (state) {
+		case PW_STREAM_STATE_UNCONNECTED:
+			pw_log_info("stream disconnected, unloading");
+			pw_impl_module_schedule_destroy(impl->module);
+			break;
+		case PW_STREAM_STATE_ERROR:
+			pw_log_error("stream error: %s", error);
+			break;
+		case PW_STREAM_STATE_STREAMING:
+			if ((errno = -session_start(impl, sess)) < 0)
+				pw_log_error("failed to start RTP stream: %m");
+			break;
+		case PW_STREAM_STATE_PAUSED:
+			if (!impl->always_process)
+				session_stop(impl, sess);
+		  break;
+		default:
+			break;
+	}
+}
+
+static const struct pw_stream_events out_stream_events = {
+	PW_VERSION_STREAM_EVENTS,
+	.destroy = stream_destroy,
+	.state_changed = on_stream_state_changed,
+	.io_changed = stream_io_changed,
+	.process = stream_process
+};
+
 static int session_new(struct impl *impl, struct sdp_info *info)
 {
 	struct session *session;
@@ -547,7 +673,8 @@ static int session_new(struct impl *impl, struct sdp_info *info)
 	uint32_t n_params;
 	uint8_t buffer[1024];
 	struct pw_properties *props;
-	int res, fd;
+	int res, sess_latency_msec;
+	const char *str;
 
 	if (impl->n_sessions >= MAX_SESSIONS) {
 		pw_log_warn("too many sessions (%u >= %u)", impl->n_sessions, MAX_SESSIONS);
@@ -559,12 +686,7 @@ static int session_new(struct impl *impl, struct sdp_info *info)
 		return -errno;
 
 	session->info = *info;
-
-	session->target_buffer = msec_to_bytes(info, impl->sess_latency_msec);
-	session->max_error = msec_to_bytes(info, ERROR_MSEC);
-
-	spa_dll_init(&session->dll);
-	spa_dll_set_bw(&session->dll, SPA_DLL_BW_MIN, 128, session->info.info.rate);
+	session->first = true;
 
 	props = pw_properties_copy(impl->stream_props);
 	if (props == NULL) {
@@ -572,9 +694,6 @@ static int session_new(struct impl *impl, struct sdp_info *info)
 		goto error;
 	}
 
-	pw_properties_setf(props, PW_KEY_NODE_RATE, "1/%d", info->info.rate);
-	pw_properties_setf(props, PW_KEY_NODE_LATENCY, "%d/%d",
-			session->target_buffer / (2 * info->stride), info->info.rate);
 	pw_properties_set(props, "rtp.origin", info->origin);
 	pw_properties_setf(props, "rtp.payload", "%u", info->payload);
 	pw_properties_setf(props, "rtp.fmt", "%s/%u/%u", info->format_info->mime,
@@ -583,11 +702,50 @@ static int session_new(struct impl *impl, struct sdp_info *info)
 		pw_properties_set(props, "rtp.session", info->session);
 		pw_properties_setf(props, PW_KEY_MEDIA_NAME, "RTP Stream (%s)",
 				info->session);
+		pw_properties_setf(props, PW_KEY_NODE_NAME, "%s",
+				info->session);
 	} else {
 		pw_properties_set(props, PW_KEY_MEDIA_NAME, "RTP Stream");
 	}
+	pw_properties_setf(props, "rtp.ts-offset", "%u", info->ts_offset);
+	pw_properties_set(props, "rtp.ts-refclk", info->refclk);
 
-	pw_log_info("new session %s %s", info->origin, info->session);
+	if ((str = pw_properties_get(impl->props, "stream.rules")) != NULL) {
+		struct session_info sinfo = {
+			.session = session,
+			.props = props,
+		};
+		pw_conf_match_rules(str, strlen(str), NAME, &props->dict,
+				rule_matched, &sinfo);
+
+		if (!sinfo.matched) {
+			res = 0;
+			pw_log_info("session '%s' was not matched", info->session);
+			goto error;
+		}
+	}
+	session->direct_timestamp = pw_properties_get_bool(props, "sess.ts-direct", false);
+
+	pw_log_info("new session %s %s direct:%d", info->origin, info->session,
+			session->direct_timestamp);
+
+	sess_latency_msec = pw_properties_get_uint32(props,
+			"sess.latency.msec", impl->sess_latency_msec);
+
+	session->target_buffer = msec_to_samples(info, sess_latency_msec);
+	session->max_error = msec_to_samples(info, ERROR_MSEC);
+
+	pw_properties_setf(props, PW_KEY_NODE_RATE, "1/%d", info->info.rate);
+	pw_properties_setf(props, PW_KEY_NODE_LATENCY, "%d/%d",
+			session->target_buffer / 2, info->info.rate);
+
+	spa_dll_init(&session->dll);
+	spa_dll_set_bw(&session->dll, SPA_DLL_BW_MIN, 128, session->info.info.rate);
+
+	if (info->channelmap[0]) {
+		pw_properties_set(props, PW_KEY_NODE_CHANNELNAMES, info->channelmap);
+		pw_log_info("channelmap: %s", info->channelmap);
+	}
 
 	session->stream = pw_stream_new(impl->core,
 			"rtp-source playback", props);
@@ -617,21 +775,10 @@ static int session_new(struct impl *impl, struct sdp_info *info)
 		goto error;
 	}
 
-	if ((fd = make_socket((const struct sockaddr *)&info->sa,
-					info->salen, impl->ifname)) < 0) {
-		res = fd;
+	if (impl->always_process &&
+		(res = session_start(impl, session)) < 0)
 		goto error;
-	}
 
-	session->source = pw_loop_add_io(impl->data_loop, fd,
-				SPA_IO_IN, true, on_rtp_io, session);
-	if (session->source == NULL) {
-		res = -errno;
-		pw_log_error("can't create io source: %m");
-		goto error;
-	}
-
-	pw_log_info("starting RTP listener");
 	session_touch(session);
 
 	session->impl = impl;
@@ -716,7 +863,32 @@ static int parse_sdp_m(struct impl *impl, char *c, struct sdp_info *info)
 	return 0;
 }
 
-static int parse_sdp_a(struct impl *impl, char *c, struct sdp_info *info)
+// some AES67 devices have channelmap encoded in i=*
+// if `i` record is found, it matches the template
+// and channel count matches, name the channels respectively
+// `i=2 channels: 01, 08` is the format
+static int parse_sdp_i(struct impl *impl, char *c, struct sdp_info *info)
+{
+	if (!strstr(c, " channels: ")) {
+		return 0;
+	}
+
+	c += strlen("i=");
+	c[strcspn(c, " ")] = '\0';
+
+	uint32_t channels;
+	if (sscanf(c, "%u", &channels) != 1 || channels <= 0 || channels > SPA_AUDIO_MAX_CHANNELS)
+		return 0;
+
+	c += strcspn(c, "\0");
+	c += strlen(" channels: ");
+
+	strncpy(info->channelmap, c, sizeof(info->channelmap) - 1);
+
+	return 0;
+}
+
+static int parse_sdp_a_rtpmap(struct impl *impl, char *c, struct sdp_info *info)
 {
 	int payload, len, rate, channels;
 
@@ -748,6 +920,7 @@ static int parse_sdp_a(struct impl *impl, char *c, struct sdp_info *info)
 	if (sscanf(c, "%u/%u", &rate, &channels) == 2) {
 		info->info.rate = rate;
 		info->info.channels = channels;
+		pw_log_debug("rate: %d, ch: %d", rate, channels);
 		if (channels == 2) {
 			info->info.position[0] = SPA_AUDIO_CHANNEL_FL;
 			info->info.position[1] = SPA_AUDIO_CHANNEL_FR;
@@ -763,6 +936,35 @@ static int parse_sdp_a(struct impl *impl, char *c, struct sdp_info *info)
 	return 0;
 }
 
+static int parse_sdp_a_mediaclk(struct impl *impl, char *c, struct sdp_info *info)
+{
+	if (!spa_strstartswith(c, "a=mediaclk:"))
+		return 0;
+
+	c += strlen("a=mediaclk:");
+
+	if (spa_strstartswith(c, "direct=")) {
+		int offset;
+		c += strlen("direct=");
+		if (sscanf(c, "%i", &offset) != 1)
+			return -EINVAL;
+		info->ts_offset = offset;
+	} else if (spa_strstartswith(c, "sender")) {
+		info->ts_offset = 0;
+	}
+	return 0;
+}
+
+static int parse_sdp_a_ts_refclk(struct impl *impl, char *c, struct sdp_info *info)
+{
+	if (!spa_strstartswith(c, "a=ts-refclk:"))
+		return 0;
+
+	c += strlen("a=ts-refclk:");
+	snprintf(info->refclk, sizeof(info->refclk), "%s", c);
+	return 0;
+}
+
 static int parse_sdp(struct impl *impl, char *sdp, struct sdp_info *info)
 {
 	char *s = sdp;
@@ -787,8 +989,14 @@ static int parse_sdp(struct impl *impl, char *sdp, struct sdp_info *info)
 			res = parse_sdp_c(impl, s, info);
 		else if (spa_strstartswith(s, "m="))
 			res = parse_sdp_m(impl, s, info);
-		else if (spa_strstartswith(s, "a="))
-			res = parse_sdp_a(impl, s, info);
+		else if (spa_strstartswith(s, "a=rtpmap:"))
+			res = parse_sdp_a_rtpmap(impl, s, info);
+		else if (spa_strstartswith(s, "a=mediaclk:"))
+			res = parse_sdp_a_mediaclk(impl, s, info);
+		else if (spa_strstartswith(s, "a=ts-refclk:"))
+			res = parse_sdp_a_ts_refclk(impl, s, info);
+		else if (spa_strstartswith(s, "i="))
+			res = parse_sdp_i(impl, s, info);
 
 		if (res < 0)
 			goto error;
@@ -938,11 +1146,20 @@ static void on_timer_event(void *data, uint64_t expirations)
 
 	clock_gettime(CLOCK_MONOTONIC, &now);
 	timestamp = SPA_TIMESPEC_TO_NSEC(&now);
-	interval = CLEANUP_INTERVAL_SEC * SPA_NSEC_PER_SEC;
+	interval = impl->cleanup_interval * SPA_NSEC_PER_SEC;
 
 	spa_list_for_each_safe(sess, tmp, &impl->sessions, link) {
-		if (sess->timestamp + interval < timestamp)
-			session_free(sess);
+		if (sess->timestamp + interval < timestamp) {
+			pw_log_debug("More than %lu elapsed from last advertisement at %lu",
+					interval, sess->timestamp);
+			if (!sess->receiving) {
+				pw_log_info("SAP timeout, closing inactive RTP source");
+				session_free(sess);
+			} else {
+				pw_log_info("SAP timeout, keeping active RTP source");
+			}
+		}
+		sess->receiving = false;
 	}
 }
 
@@ -1013,7 +1230,6 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 {
 	struct pw_context *context = pw_impl_module_get_context(module);
 	struct impl *impl;
-	struct pw_properties *props = NULL, *stream_props = NULL;
 	const char *str;
 	struct timespec value, interval;
 	int res = 0;
@@ -1024,52 +1240,49 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if (impl == NULL)
 		return -errno;
 
+	spa_list_init(&impl->sessions);
+
 	if (args == NULL)
 		args = "";
 
-	props = pw_properties_new_string(args);
-	if (props == NULL) {
+	impl->props = pw_properties_new_string(args);
+	impl->stream_props = pw_properties_new(NULL, NULL);
+	if (impl->props == NULL || impl->stream_props == NULL) {
 		res = -errno;
 		pw_log_error( "can't create properties: %m");
 		goto out;
 	}
-	spa_list_init(&impl->sessions);
-	impl->props = props;
-
-	stream_props = pw_properties_new(NULL, NULL);
-	if (stream_props == NULL) {
-		res = -errno;
-		pw_log_error( "can't create properties: %m");
-		goto out;
-	}
-	impl->stream_props = stream_props;
 
 	impl->module = module;
 	impl->module_context = context;
 	impl->loop = pw_context_get_main_loop(context);
 	impl->data_loop = pw_data_loop_get_loop(pw_context_get_data_loop(context));
 
-	if (pw_properties_get(stream_props, PW_KEY_NODE_VIRTUAL) == NULL)
-		pw_properties_set(stream_props, PW_KEY_NODE_VIRTUAL, "true");
-	if (pw_properties_get(stream_props, PW_KEY_NODE_NETWORK) == NULL)
-		pw_properties_set(stream_props, PW_KEY_NODE_NETWORK, "true");
+	if (pw_properties_get(impl->stream_props, PW_KEY_NODE_VIRTUAL) == NULL)
+		pw_properties_set(impl->stream_props, PW_KEY_NODE_VIRTUAL, "true");
+	if (pw_properties_get(impl->stream_props, PW_KEY_NODE_NETWORK) == NULL)
+		pw_properties_set(impl->stream_props, PW_KEY_NODE_NETWORK, "true");
 
-	if ((str = pw_properties_get(props, "stream.props")) != NULL)
-		pw_properties_update_string(stream_props, str, strlen(str));
+	if ((str = pw_properties_get(impl->props, "stream.props")) != NULL)
+		pw_properties_update_string(impl->stream_props, str, strlen(str));
 
-	str = pw_properties_get(props, "local.ifname");
+	str = pw_properties_get(impl->props, "local.ifname");
 	impl->ifname = str ? strdup(str) : NULL;
 
-	str = pw_properties_get(props, "sap.ip");
+	impl->always_process = pw_properties_get_bool(impl->props, PW_KEY_NODE_ALWAYS_PROCESS, false);
+
+	str = pw_properties_get(impl->props, "sap.ip");
 	impl->sap_ip = strdup(str ? str : DEFAULT_SAP_IP);
-	impl->sap_port = pw_properties_get_uint32(props,
+	impl->sap_port = pw_properties_get_uint32(impl->props,
 			"sap.port", DEFAULT_SAP_PORT);
-	impl->sess_latency_msec = pw_properties_get_uint32(props,
+	impl->sess_latency_msec = pw_properties_get_uint32(impl->props,
 			"sess.latency.msec", DEFAULT_SESS_LATENCY);
+	impl->cleanup_interval = pw_properties_get_uint32(impl->props,
+			"sap.interval.sec", DEFAULT_CLEANUP_INTERVAL_SEC);
 
 	impl->core = pw_context_get_object(impl->module_context, PW_TYPE_INTERFACE_Core);
 	if (impl->core == NULL) {
-		str = pw_properties_get(props, PW_KEY_REMOTE_NAME);
+		str = pw_properties_get(impl->props, PW_KEY_REMOTE_NAME);
 		impl->core = pw_context_connect(impl->module_context,
 				pw_properties_new(
 					PW_KEY_REMOTE_NAME, str,
@@ -1098,7 +1311,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	}
 	value.tv_sec = 0;
 	value.tv_nsec = 1;
-	interval.tv_sec = CLEANUP_INTERVAL_SEC;
+	interval.tv_sec = impl->cleanup_interval;
 	interval.tv_nsec = 0;
 	pw_loop_update_timer(impl->loop, impl->timer, &value, &interval, false);
 
