@@ -64,6 +64,9 @@ static void * registry_bind(void *object, uint32_t id,
 	if (!PW_PERM_IS_R(permissions))
 		goto error_no_id;
 
+	if (resource->client->recv_generation != 0 && global->generation > resource->client->recv_generation)
+		goto error_stale_id;
+
 	if (!spa_streq(global->type, type))
 		goto error_wrong_interface;
 
@@ -75,6 +78,12 @@ static void * registry_bind(void *object, uint32_t id,
 
 	return NULL;
 
+error_stale_id:
+	pw_log_debug("registry %p: not binding stale global "
+			"id %u to %u, generation:%"PRIu64" recv-generation:%"PRIu64,
+			resource, id, new_id, global->generation, resource->client->recv_generation);
+	pw_resource_errorf_id(resource, new_id, -ESTALE, "no global %u any more", id);
+	goto error_exit_clean;
 error_no_id:
 	pw_log_debug("registry %p: no global with id %u to bind to %u", resource, id, new_id);
 	pw_resource_errorf_id(resource, new_id, -ENOENT, "no global %u", id);
@@ -110,7 +119,10 @@ static int registry_destroy(void *object, uint32_t id)
 	if (!PW_PERM_IS_R(permissions))
 		goto error_no_id;
 
-	if (id == PW_ID_CORE || !PW_PERM_CHECK(permissions, PW_PERM_X|PW_PERM_W))
+	if (resource->client->recv_generation != 0 && global->generation > resource->client->recv_generation)
+		goto error_stale_id;
+
+	if (id == PW_ID_CORE || !PW_PERM_IS_X(permissions))
 		goto error_not_allowed;
 
 	pw_log_debug("global %p: destroy global id %d", global, id);
@@ -118,6 +130,13 @@ static int registry_destroy(void *object, uint32_t id)
 	pw_global_destroy(global);
 	return 0;
 
+error_stale_id:
+	pw_log_debug("registry %p: not destroying stale global "
+			"id %u, generation:%"PRIu64" recv-generation:%"PRIu64,
+			resource, id, global->generation, resource->client->recv_generation);
+	pw_resource_errorf(resource, -ESTALE, "no global %u any more", id);
+	res = -ESTALE;
+	goto error_exit;
 error_no_id:
 	pw_log_debug("registry %p: no global with id %u to destroy", resource, id);
 	pw_resource_errorf(resource, -ENOENT, "no global %u", id);
@@ -138,9 +157,9 @@ static const struct pw_registry_methods registry_methods = {
 	.destroy = registry_destroy
 };
 
-static void destroy_registry_resource(void *object)
+static void destroy_registry_resource(void *_data)
 {
-	struct resource_data *data = object;
+	struct resource_data *data = _data;
 	struct pw_resource *resource = data->resource;
 	spa_list_remove(&resource->link);
 	spa_hook_remove(&data->resource_listener);
@@ -332,7 +351,7 @@ core_create_object(void *object,
 	if (obj == NULL)
 		goto error_create_failed;
 
-	return 0;
+	return obj;
 
 error_no_factory:
 	res = -ENOENT;
@@ -488,13 +507,13 @@ static const struct pw_resource_events core_resource_events = {
 };
 
 static int
-global_bind(void *_data,
+global_bind(void *object,
 	    struct pw_impl_client *client,
 	    uint32_t permissions,
 	    uint32_t version,
 	    uint32_t id)
 {
-	struct pw_impl_core *this = _data;
+	struct pw_impl_core *this = object;
 	struct pw_global *global = this->global;
 	struct pw_resource *resource;
 	struct resource_data *data;
@@ -536,9 +555,9 @@ error:
 	return res;
 }
 
-static void global_destroy(void *object)
+static void global_destroy(void *data)
 {
-	struct pw_impl_core *core = object;
+	struct pw_impl_core *core = data;
 	spa_hook_remove(&core->global_listener);
 	core->global = NULL;
 	pw_impl_core_destroy(core);
