@@ -26,24 +26,24 @@
 #include <errno.h>
 #include <sys/time.h>
 
+#include <spa/support/thread.h>
 #include <spa/utils/result.h>
 
 #include "log.h"
+#include "thread.h"
 #include "thread-loop.h"
 
-#define NAME "thread-loop"
+PW_LOG_TOPIC_EXTERN(log_thread_loop);
+#define PW_LOG_TOPIC_DEFAULT log_thread_loop
+
 
 #define pw_thread_loop_events_emit(o,m,v,...) spa_hook_list_call(&o->listener_list, struct pw_thread_loop_events, m, v, ##__VA_ARGS__)
 #define pw_thread_loop_events_destroy(o)	pw_thread_loop_events_emit(o, destroy, 0)
 
-#ifdef __FreeBSD__
-#define pthread_setname_np pthread_set_name_np
-#endif
-
 /** \cond */
 struct pw_thread_loop {
 	struct pw_loop *loop;
-	char *name;
+	char name[16];
 
 	struct spa_hook_list listener_list;
 
@@ -91,7 +91,7 @@ static void do_stop(void *data, uint64_t count)
 
 #define CHECK(expression,label)						\
 do {									\
-	if ((errno = expression) != 0) {				\
+	if ((errno = (expression)) != 0) {				\
 		res = -errno;						\
 		pw_log_error(#expression ": %s", strerror(errno));	\
 		goto label;						\
@@ -111,7 +111,7 @@ static struct pw_thread_loop *loop_new(struct pw_loop *loop,
 	if (this == NULL)
 		return NULL;
 
-	pw_log_debug(NAME" %p: new", this);
+	pw_log_debug("%p: new name:%s", this, name);
 
 	if (loop == NULL) {
 		loop = pw_loop_new(props);
@@ -122,7 +122,7 @@ static struct pw_thread_loop *loop_new(struct pw_loop *loop,
 		goto clean_this;
 	}
 	this->loop = loop;
-	this->name = name ? strdup(name) : NULL;
+	snprintf(this->name, sizeof(this->name), "%s", name ? name : "pw-thread-loop");
 
 	spa_hook_list_init(&this->listener_list);
 
@@ -154,7 +154,6 @@ clean_lock:
 clean_this:
 	if (this->created && this->loop)
 		pw_loop_destroy(this->loop);
-	free(this->name);
 	free(this);
 	errno = -res;
 	return NULL;
@@ -162,17 +161,16 @@ clean_this:
 
 /** Create a new \ref pw_thread_loop
  *
- * \param loop the loop to wrap
  * \param name the name of the thread or NULL
+ * \param props a dict of properties for the thread loop
  * \return a newly allocated \ref  pw_thread_loop
  *
- * Make a new \ref pw_thread_loop that will run \a loop in
+ * Make a new \ref pw_thread_loop that will run in
  * a thread with \a name.
  *
  * After this function you should probably call pw_thread_loop_start() to
  * actually start the thread
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 struct pw_thread_loop *pw_thread_loop_new(const char *name,
@@ -181,6 +179,20 @@ struct pw_thread_loop *pw_thread_loop_new(const char *name,
 	return loop_new(NULL, name, props);
 }
 
+/** Create a new \ref pw_thread_loop
+ *
+ * \param loop the loop to wrap
+ * \param name the name of the thread or NULL
+ * \param props a dict of properties for the thread loop
+ * \return a newly allocated \ref  pw_thread_loop
+ *
+ * Make a new \ref pw_thread_loop that will run \a loop in
+ * a thread with \a name.
+ *
+ * After this function you should probably call pw_thread_loop_start() to
+ * actually start the thread
+ *
+ */
 SPA_EXPORT
 struct pw_thread_loop *pw_thread_loop_new_full(struct pw_loop *loop,
 		const char *name, const struct spa_dict *props)
@@ -188,7 +200,7 @@ struct pw_thread_loop *pw_thread_loop_new_full(struct pw_loop *loop,
 	return loop_new(loop, name, props);
 }
 
-/** Destroy a threaded loop \memberof pw_thread_loop */
+/** Destroy a threaded loop */
 SPA_EXPORT
 void pw_thread_loop_destroy(struct pw_thread_loop *loop)
 {
@@ -209,7 +221,6 @@ void pw_thread_loop_destroy(struct pw_thread_loop *loop)
 	pthread_cond_destroy(&loop->cond);
 	pthread_mutex_destroy(&loop->lock);
 
-	free(loop->name);
 	free(loop);
 }
 
@@ -235,19 +246,18 @@ static void *do_loop(void *user_data)
 	int res;
 
 	pthread_mutex_lock(&this->lock);
-	pw_log_debug(NAME" %p: enter thread", this);
-	pthread_setname_np(this->thread, this->name ? this->name : "pipewire-thread");
+	pw_log_debug("%p: enter thread", this);
 	pw_loop_enter(this->loop);
 
 	while (this->running) {
 		if ((res = pw_loop_iterate(this->loop, -1)) < 0) {
 			if (res == -EINTR)
 				continue;
-			pw_log_warn(NAME" %p: iterate error %d (%s)",
+			pw_log_warn("%p: iterate error %d (%s)",
 					this, res, spa_strerror(res));
 		}
 	}
-	pw_log_debug(NAME" %p: leave thread", this);
+	pw_log_debug("%p: leave thread", this);
 	pw_loop_leave(this->loop);
 	pthread_mutex_unlock(&this->lock);
 
@@ -259,69 +269,76 @@ static void *do_loop(void *user_data)
  * \param loop a \ref pw_thread_loop
  * \return 0 on success
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 int pw_thread_loop_start(struct pw_thread_loop *loop)
 {
+	int err;
+
 	if (!loop->running) {
-		int err;
+		struct spa_thread *thr;
+		struct spa_dict_item items[1];
 
 		loop->running = true;
-		if ((err = pthread_create(&loop->thread, NULL, do_loop, loop)) != 0) {
-			pw_log_warn(NAME" %p: can't create thread: %s", loop,
-				    strerror(err));
-			loop->running = false;
-			return -err;
-		}
+
+		items[0] = SPA_DICT_ITEM_INIT(SPA_KEY_THREAD_NAME, loop->name);
+		thr = pw_thread_utils_create(&SPA_DICT_INIT_ARRAY(items), do_loop, loop);
+		if (thr == NULL)
+			goto error;
+
+		loop->thread = (pthread_t)thr;
 	}
 	return 0;
+
+error:
+	err = errno;
+	pw_log_warn("%p: can't create thread: %s", loop,
+		    strerror(err));
+	loop->running = false;
+	return -err;
 }
 
 /** Quit the loop and stop its thread
  *
  * \param loop a \ref pw_thread_loop
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 void pw_thread_loop_stop(struct pw_thread_loop *loop)
 {
-	pw_log_debug(NAME": %p stopping %d", loop, loop->running);
+	pw_log_debug("%p stopping %d", loop, loop->running);
 	if (loop->running) {
-		pw_log_debug(NAME": %p signal", loop);
+		pw_log_debug("%p signal", loop);
 		pw_loop_signal_event(loop->loop, loop->event);
-		pw_log_debug(NAME": %p join", loop);
+		pw_log_debug("%p join", loop);
 		pthread_join(loop->thread, NULL);
-		pw_log_debug(NAME": %p joined", loop);
+		pw_log_debug("%p joined", loop);
 		loop->running = false;
 	}
-	pw_log_debug(NAME": %p stopped", loop);
+	pw_log_debug("%p stopped", loop);
 }
 
 /** Lock the mutex associated with \a loop
  *
  * \param loop a \ref pw_thread_loop
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 void pw_thread_loop_lock(struct pw_thread_loop *loop)
 {
 	pthread_mutex_lock(&loop->lock);
-	pw_log_trace(NAME": %p", loop);
+	pw_log_trace("%p", loop);
 }
 
 /** Unlock the mutex associated with \a loop
  *
  * \param loop a \ref pw_thread_loop
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 void pw_thread_loop_unlock(struct pw_thread_loop *loop)
 {
-	pw_log_trace(NAME": %p", loop);
+	pw_log_trace("%p", loop);
 	pthread_mutex_unlock(&loop->lock);
 }
 
@@ -333,12 +350,11 @@ void pw_thread_loop_unlock(struct pw_thread_loop *loop)
  * Signal the thread of \a loop. If \a wait_for_accept is true,
  * this function waits until \ref pw_thread_loop_accept() is called.
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 void pw_thread_loop_signal(struct pw_thread_loop *loop, bool wait_for_accept)
 {
-	pw_log_trace(NAME": %p, waiting:%d accept:%d",
+	pw_log_trace("%p, waiting:%d accept:%d",
 			loop, loop->n_waiting, wait_for_accept);
 	if (loop->n_waiting > 0)
 		pthread_cond_broadcast(&loop->cond);
@@ -355,16 +371,15 @@ void pw_thread_loop_signal(struct pw_thread_loop *loop, bool wait_for_accept)
  *
  * \param loop a \ref pw_thread_loop to signal
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 void pw_thread_loop_wait(struct pw_thread_loop *loop)
 {
-	pw_log_trace(NAME": %p, waiting %d", loop, loop->n_waiting);
+	pw_log_trace("%p, waiting %d", loop, loop->n_waiting);
 	loop->n_waiting++;
 	pthread_cond_wait(&loop->cond, &loop->lock);
 	loop->n_waiting--;
-	pw_log_trace(NAME": %p, waiting done %d", loop, loop->n_waiting);
+	pw_log_trace("%p, waiting done %d", loop, loop->n_waiting);
 }
 
 /** Wait for the loop thread to call \ref pw_thread_loop_signal()
@@ -374,7 +389,6 @@ void pw_thread_loop_wait(struct pw_thread_loop *loop)
  * \param wait_max_sec the maximum number of seconds to wait for a \ref pw_thread_loop_signal()
  * \return 0 on success or ETIMEDOUT on timeout or a negative errno value.
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 int pw_thread_loop_timed_wait(struct pw_thread_loop *loop, int wait_max_sec)
@@ -396,7 +410,6 @@ int pw_thread_loop_timed_wait(struct pw_thread_loop *loop, int wait_max_sec)
  * \param timeout the time in nanoseconds to add to \a tp
  * \return 0 on success or a negative errno value on error.
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 int pw_thread_loop_get_time(struct pw_thread_loop *loop, struct timespec *abstime, int64_t timeout)
@@ -420,10 +433,9 @@ int pw_thread_loop_get_time(struct pw_thread_loop *loop, struct timespec *abstim
  * \param abstime the absolute time to wait for a \ref pw_thread_loop_signal()
  * \return 0 on success or -ETIMEDOUT on timeout or a negative error value
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
-int pw_thread_loop_timed_wait_full(struct pw_thread_loop *loop, struct timespec *abstime)
+int pw_thread_loop_timed_wait_full(struct pw_thread_loop *loop, const struct timespec *abstime)
 {
 	int ret;
 	loop->n_waiting++;
@@ -436,7 +448,6 @@ int pw_thread_loop_timed_wait_full(struct pw_thread_loop *loop, struct timespec
  *
  * \param loop a \ref pw_thread_loop to signal
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 void pw_thread_loop_accept(struct pw_thread_loop *loop)
@@ -450,10 +461,9 @@ void pw_thread_loop_accept(struct pw_thread_loop *loop)
  * \param loop a \ref pw_thread_loop to signal
  * \return true when called inside the thread of \a loop.
  *
- * \memberof pw_thread_loop
  */
 SPA_EXPORT
 bool pw_thread_loop_in_thread(struct pw_thread_loop *loop)
 {
-	return pthread_self() == loop->thread;
+	return loop->running && pthread_self() == loop->thread;
 }
